# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import argparse
import os
import re
import shlex

from taskgraph.decision import PER_PROJECT_PARAMETERS
from taskgraph.parameters import extend_parameters_schema
from taskgraph.util.vcs import get_repository
from voluptuous import Any, Optional, Required


TRY_SYNTAX_RE = re.compile(r"\btry:\s*(.*)\s*$", re.M)

def parse_try_syntax(message):
    parser = argparse.ArgumentParser()
    parser.add_argument("--nspr-patch", action="store_true")
    parser.add_argument("-b", "--build", default="do")
    parser.add_argument("-p", "--platform", default="all")
    parser.add_argument("-u", "--unittests", default="none")
    parser.add_argument("-t", "--tools", default="none")
    parser.add_argument("-e", "--extra-builds")
    match = TRY_SYNTAX_RE.search(message)
    if not match:
        return
    args = shlex.split(match.group(1))
    opts = parser.parse_args(args)
    builds = [t for t in opts.build if t in ["d", "o"]]
    # If the given value is nonsense default to debug and opt builds.
    if not builds:
        builds = ["d", "o"]

    platforms = opts.platform.split(",")

    unittests = opts.unittests.split(",")
    if "gtests" in unittests:
        unittests.append("gtest")

    tools = opts.tools.split(",")
    return {
        "nspr_patch": opts.nspr_patch,
        "builds": builds,
        "platforms": platforms,
        "unittests": unittests,
        "tools": tools,
        "extra": opts.extra_builds == "all",
    }


def decision_parameters(graph_config, parameters):
    repo_path = os.getcwd()
    repo = get_repository(repo_path)
    try:
        commit_message = repo.get_commit_message()
    except UnicodeDecodeError:
        commit_message = ""
    parameters["try_options"] = {}
    if parameters["project"] != "nss-try":
        return
    args = parse_try_syntax(commit_message)
    if args:
       parameters["try_options"] = args


def default_parameters(repo_root):
    return {
        "try_options": {},
    }


def register(graph_config):
    schema = {
        Optional("try_options"): {
            Optional("nspr_patch"): bool,
            Optional("builds"): [Any("d", "o")],
            Optional("platforms"): [str],
            Optional("unittests"): [str],
            Optional("tools"): [str],
            Optional("extra"): bool,
        },
    }
    extend_parameters_schema(schema, default_parameters)

    from . import target_tasks

    PER_PROJECT_PARAMETERS["nss-try"] = {"target_tasks_method": "nss_try_tasks"}




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from taskgraph.transforms.base import TransformSequence

transforms = TransformSequence()

@transforms.add
def set_treeherder_symbol(config, tasks):
    for task in tasks:
        dep = config.kind_dependencies_tasks[task["dependencies"][task["attributes"]["primary-kind-dependency"]]]
        if dep.task["extra"]["treeherder"].get("groupSymbol"):
            task["treeherder"]["symbol"] = f'{dep.task["extra"]["treeherder"].get("groupSymbol")}({task["treeherder"]["symbol"]})'
        task["treeherder"]["platform"] = f'{task["attributes"]["build_platform"]}/{task["attributes"]["build_type"]}'
        yield task



#!/usr/bin/env python3
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import subprocess
import os
import sys

def main():
    args = [os.path.realpath(x) for x in sys.argv[1:]]
    script = os.path.dirname(os.path.abspath(__file__))+'/certdata.perl'
    subprocess.check_call([os.environ.get('PERL', 'perl'), script] + args,
                          env=os.environ)

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.

import json
import os
import subprocess
import hashlib

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives import serialization
import binascii

script_dir = os.path.dirname(os.path.abspath(__file__))

# Imports a JSON testvector file.
def import_testvector(file):
    """Import a JSON testvector file and return an array of the contained objects."""
    with open(file) as f:
        vectors = json.loads(f.read())
    return vectors

# Convert a test data string to a hex array.
def string_to_hex(string):
    """Convert a string of hex chars to a string representing a C-format array of hex bytes."""
    b = bytearray.fromhex(string)
    result = ', '.join("{:#04x}".format(x) for x in b)
    return result

def digest(string):
    b = bytearray.fromhex(string)
    h = hashlib.sha3_256()
    h.update(b)
    return h.hexdigest()

#now split them into readable lines
def string_line_split(string, group_len, count, spaces):
    result=''
    lstring = string;
    while (len(lstring) > count*group_len):
        result += lstring[:count*group_len].rjust(spaces);
        result += '\n'
        lstring=lstring[count*group_len:];
    result += lstring.rjust(spaces)
    return result

# put it together
def string_to_hex_array(string):
    result="{\n"
    result += string_line_split(' '+string_to_hex(string), 6, 12, 4)
    result += '}'
    return result


mldsa_spki={
    "ML-DSA-44": "30820532300b06096086480165030403110382052100",
    "ML-DSA-65": "308207b2300b0609608648016503040312038207a100",
    "ML-DSA-87": "30820a32300b060960864801650304031303820a2100",
}

mldsa_p11param={
    "ML-DSA-44": "CKP_ML_DSA_44",
    "ML-DSA-65": "CKP_ML_DSA_65",
    "ML-DSA-87": "CKP_ML_DSA_87",
}

class MLDSA_VERIFY():
    key_name=''
    has_name=False
    def init_global(self, group, group_result, out_defs):
        paramSet= group['parameterSet']
        self.paramSet=paramSet
        if not paramSet in mldsa_spki:
            return False
        if 'pk' in group:
            rawkey= group['pk']
            paramSet= group['parameterSet']
            key_name = "kPubKey"+str(group['tgId'])
            key=mldsa_spki[paramSet]+rawkey
            out_defs.append('// Key Type'+paramSet+'\n')
            out_defs.append('static const std::vector<uint8_t> ' + key_name + string_to_hex_array(key) + ';\n\n')
            self.key_name=key_name
            self.has_name=True
            return True
        self.has_name=False
        # only fetch the test types that we support: external
        # pure, and externalMu == false.
        # we can adjust these when we and externalMu support and
        # MLDSA Hash support (if we ever do)
        if 'signatureInterface' in group:
            if group['signatureInterface'] != "external":
                return False
        if 'preHash' in group:
            if group['preHash'] != "pure":
                return False
        if 'externalMu' in group:
            if group['externalMu']:
                return False
        return True
    def format_testcase(self, testcase, testcase_result, out_defs):
        key=mldsa_spki[self.paramSet]+testcase['pk']
        result = '\n// {}\n'.format(self.paramSet)
        result = '// tcID: {}\n'.format(testcase['tcId'])
        result += '{{{},\n'.format(testcase['tcId'])
        result += '// signature\n{},\n'.format(string_to_hex_array(testcase['signature']))
        if self.has_name:
            result += '{},\n'.format(self.key_name)
        else:
            result += '// pubkey\n{},\n'.format(string_to_hex_array(key))
        if 'context' in testcase:
            result += '// context\n{},\n'.format(string_to_hex_array(testcase['context']))
        else:
            result += '// context\n{{}},\n'
        result += '// message\n{},\n'.format(string_to_hex_array(testcase['message']))
        result += '{}}},\n'.format(str(testcase_result['testPassed']).lower())
        display = 'tcID {}: {} '.format(testcase['tcId'],self.paramSet)
        display += 'key({}) message({}) '.format(len(key)/2,len(testcase['message'])/2)
        display += 'signature({}) ctx({}) '.format(len(testcase['signature'])/2,len(testcase['context'])/2,)
        display += '{}'.format(testcase_result['testPassed'])
        print(display)
        return result

class MLDSA_KEYGEN():
    def init_global(self, group, group_result, out_defs):
        paramSet= group['parameterSet']
        self.paramSet = paramSet
        return paramSet in mldsa_p11param
    def format_testcase(self, testcase, testcase_result, out_defs):
        seed=testcase['seed']
        pubk=testcase_result['pk']
        privk=testcase_result['sk']
        result = '\n// {}\n'.format(self.paramSet)
        result = '// tcID: {}\n'.format(testcase['tcId'])
        result += '{{{},\n'.format(testcase['tcId'])
        result += '{},\n'.format(mldsa_p11param[self.paramSet])
        result += '//seed\n{},\n'.format(string_to_hex_array(seed))
        result += '//raw pubkey\n{},\n'.format(string_to_hex_array(pubk))
        result += '//raw privkey\n{}}},\n'.format(string_to_hex_array(privk))
        display = 'tcID {}: {} '.format(testcase['tcId'],self.paramSet)
        display += 'seed({}) pubk({}) '.format(len(seed)/2,len(pubk)/2)
        display += 'privk({})'.format(len(privk)/2)
        print(display)
        return result

mlkem_freebl_param={
    "ML-KEM-768": "params_ml_kem768",
    "ML-KEM-1024": "params_ml_kem1024",
}

mlkem_freebl_test_param={
    "ML-KEM-768": "params_ml_kem768_test_mode",
    "ML-KEM-1024": "params_ml_kem1024_test_mode",
}

mlkem_prefix={
    "ML-KEM-768": "MlKem768",
    "ML-KEM-1024": "MlKem1024",
}

class MLKEM_KEYGEN():
    def init_global(self, group, group_result, out_defs):
        paramSet= group['parameterSet']
        self.paramSet = paramSet
        return paramSet in mlkem_freebl_param
    def format_testcase(self, testcase, testcase_result, out_defs):
        seed=testcase['d']+testcase['z']
        pubk=testcase_result['ek']
        privk=testcase_result['dk']
        pubkDigest=digest(pubk)
        privkDigest=digest(privk)
        result = '\n// {}\n'.format(self.paramSet)
        result = '// tcID: {}\n'.format(testcase['tcId'])
        result += '{{{},\n'.format(testcase['tcId'])
        result += '{},\n'.format(mlkem_freebl_param[self.paramSet])
        result += '//seed\n{},\n'.format(string_to_hex_array(seed))
        result += '//publicKeyDigest\n{},\n'.format(string_to_hex_array(pubkDigest))
        result += '//privateKeyDigest\n{}}},\n'.format(string_to_hex_array(privkDigest))
        display = 'tcID {}: {} '.format(testcase['tcId'],self.paramSet)
        display += 'seed({}) pubk({}) '.format(len(seed)/2,len(pubk)/2)
        display += 'privk({})'.format(len(privk)/2)
        print(display)
        return result

class MLKEM_ENCAP():
    key_name=''
    has_name=False
    def init_global(self, group, group_result, out_defs):
        paramSet= group['parameterSet']
        self.paramSet=paramSet
        if not paramSet in mlkem_freebl_test_param:
            return False
        if  group['function'] != 'encapsulation':
            return False
        return True
    def format_testcase(self, testcase, testcase_result, out_defs):
        key=testcase['ek']
        result = '\n// {}\n'.format(self.paramSet)
        result = '// tcID: {}\n'.format(testcase['tcId'])
        result += '{{{},\n'.format(testcase['tcId'])
        result += '{},\n'.format(mlkem_freebl_test_param[self.paramSet])
        result += '// entropy\n{},\n'.format(string_to_hex_array(testcase['m']))
        result += '// publicKey\n{},\n'.format(string_to_hex_array(testcase['ek']))
        cipherTextDigest=digest(testcase_result['c'])
        result += '// cipherTextDigest\n{},\n'.format(string_to_hex_array(cipherTextDigest))
        result += '// secret\n{},\ntrue}},\n'.format(string_to_hex_array(testcase_result['k']))
        display = 'tcID {}: {} '.format(testcase['tcId'],self.paramSet)
        display += 'key({}) encapsulate'.format(len(key)/2)
        print(display)
        return result

class MLKEM_DECAP():
    key_name=''
    has_name=False
    def init_global(self, group, group_result, out_defs):
        paramSet= group['parameterSet']
        self.paramSet=paramSet
        if not paramSet in mlkem_freebl_test_param:
            return False
        if  group['function'] != 'decapsulation':
            return False
        return True
    def format_testcase(self, testcase, testcase_result, out_defs):
        result = '\n// {}\n'.format(self.paramSet)
        result = '// tcID: {}\n'.format(testcase['tcId'])
        result += '{{{},\n'.format(testcase['tcId'])
        result += '{},\n'.format(mlkem_freebl_test_param[self.paramSet])
        result += '// privateKey\n{},\n'.format(string_to_hex_array(testcase['dk']))
        result += '// ciphertext\n{},\n'.format(string_to_hex_array(testcase['c']))
        result += '// secret\n{},\ntrue}},\n'.format(string_to_hex_array(testcase_result['k']))
        display = 'tcID {}: {} decapsulate'.format(testcase['tcId'],self.paramSet)
        print(display)
        return result

def matchID(_id, source, target):
    for i in target:
        if i[_id] == source[_id]:
            return i
    return {}

def generate_vectors_file(params):
    """
    Generate and store a .h-file with test vectors for one test.

    params -- Dictionary with parameters for test vector generation for the desired test.
    """

    cases = import_testvector(os.path.join(script_dir, params['source_dir'] + params['prompt_file']))
    result = import_testvector(os.path.join(script_dir, params['source_dir'] + params['result_file']))

    base_vectors = ""
    if 'base' in params:
        with open(os.path.join(script_dir, params['base'])) as base:
            base_vectors = base.read()
        base_vectors += "\n\n"

    header = standard_params['license']
    header += "\n"
    header += standard_params['top_comment']
    header += "\n"
    header += "#ifndef " + params['section'] + "\n"
    header += "#define " + params['section'] + "\n"
    header += "\n"

    for include in standard_params['includes']:
        header += "#include " + include + "\n"

    header += "\n"

    if 'includes' in params:
        for include in params['includes']:
            header += "#include " + include + "\n"
        header += "\n"

    shared_defs = []
    vectors_file = base_vectors + params['array_init']

    for group in cases['testGroups']:
        group_result = matchID('tgId', group, result['testGroups']);
        if (not params['formatter'].init_global(group, group_result, shared_defs)):
            continue;
        for test in group['tests']:
            test_result = matchID('tcId', test, group_result['tests']);
            vectors_file += params['formatter'].format_testcase(test, test_result, shared_defs)

    vectors_file = vectors_file[:params['crop_size_end']] + '\n};\n\n'
    vectors_file += "#endif // " + params['section'] + '\n'

    with open(os.path.join(script_dir, params['target']), 'w') as target:
        target.write(header)
        for definition in shared_defs:
            target.write(definition)
        target.write(vectors_file)


standard_params = {
    'includes': ['"testvectors_base/test-structs.h"'],
    'license':
"""/* vim: set ts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */
 """,

    'top_comment':
"""/* This file is generated from sources in nss/gtests/common/wycheproof
 * automatically and should not be touched manually.
 * Generation is trigged by calling python3 genTestVectors.py */
 """
}

# Parameters that describe the generation of a testvector file for each supoorted test.
# source -- relative path to the wycheproof JSON source file with testvectors.
# base -- relative path to non-wycheproof vectors.
# target -- relative path to where the finished .h-file is written.
# array_init -- string to initialize the c-header style array of testvectors.
# formatter -- the test case formatter class to be used for this test.
# crop_size_end -- number of characters removed from the end of the last generated test vector to close the array definition.
# section -- name of the section
# comment -- additional comments to add to the file just before definition of the test vector array.

ml_dsa_verify_params = {
    'source_dir': 'source_vectors/',
    'test_name': 'ML-DSA-sigVer-FIPS204',
    'tag': 'v1.1.0.40',
    'prompt_file': 'ml_dsa_verify_prompt.json',
    'result_file': 'ml_dsa_verify_result.json',
    'target': '../testvectors/ml-dsa-verify-vectors.h',
    'array_init': 'const MlDsaVerifyTestVector kMLDsaNISTVerifyVectors[] = {\n',
    'formatter' : MLDSA_VERIFY(),
    'crop_size_end': -2,
    'section': 'mldsa_verify_vectors_h__',
    'comment' : ''
}

ml_dsa_keygen_params = {
    'source_dir': 'source_vectors/',
    'test_name': 'ML-DSA-keyGen-FIPS204',
    'tag': 'v1.1.0.40',
    'prompt_file': 'ml_dsa_keygen_prompt.json',
    'result_file': 'ml_dsa_keygen_result.json',
    'target': '../testvectors/ml-dsa-keygen-vectors.h',
    'array_init': 'const MlDsaKeyGenTestVector kMLDsaNISTKeyGenVectors[] = {\n',
    'formatter' : MLDSA_KEYGEN(),
    'crop_size_end': -2,
    'section': 'mldsa_keygen_vectors_h__',
    'comment' : ''
}

ml_kem_decap_params = {
    'source_dir': 'source_vectors/',
    'test_name': 'ML-KEM-encapDecap-FIPS203',
    'tag': 'v1.1.0.40',
    'prompt_file': 'ml_kem_encap_decap_prompt.json',
    'result_file': 'ml_kem_encap_decap_result.json',
    'target': '../testvectors/ml-kem-decap-vectors.h',
    'array_init': 'const std::vector<MlKemDecapTestVector> MlKemDecapTests = {\n',
    'formatter' : MLKEM_DECAP(),
    'crop_size_end': -2,
    'section': 'mlkem_decap_vectors_h__',
    'comment' : ''
}

ml_kem_encap_params = {
    'source_dir': 'source_vectors/',
    'test_name': 'ML-KEM-encapDecap-FIPS203',
    'tag': 'v1.1.0.40',
    'prompt_file': 'ml_kem_encap_decap_prompt.json',
    'result_file': 'ml_kem_encap_decap_result.json',
    'target': '../testvectors/ml-kem-encap-vectors.h',
    'array_init': 'const std::vector<MlKemEncapTestVector> MlKemEncapTests = {\n',
    'formatter' : MLKEM_ENCAP(),
    'crop_size_end': -2,
    'section': 'mlkem_encap_vectors_h__',
    'comment' : ''
}

ml_kem_keygen_params = {
    'source_dir': 'source_vectors/',
    'test_name': 'ML-KEM-keyGen-FIPS203',
    'tag': 'v1.1.0.40',
    'prompt_file': 'ml_kem_keygen_prompt.json',
    'result_file': 'ml_kem_keygen_result.json',
    'target': '../testvectors/ml-kem-keygen-vectors.h',
    'array_init': 'const std::vector<MlKemKeyGenTestVector> MlKemKeyGenTests = {\n',
    'formatter' : MLKEM_KEYGEN(),
    'crop_size_end': -2,
    'section': 'mlkem_keygen_vectors_h__',
    'comment' : ''
}


def update_tests(tests):

    remote_base = "https://raw.githubusercontent.com/usnistgov/ACVP-Server/refs/tags/"
    for test in tests:
        remote = remote_base+test['tag']+"/gen-val/json-files/"+test['test_name']+"/"
        subprocess.check_call(['wget', remote+"/prompt.json", '-O',
                               script_dir+'/'+test['source_dir']+test['prompt_file']])
        subprocess.check_call(['wget', remote+"/expectedResults.json", '-O',
                               script_dir+'/'+test['source_dir']+test['result_file']])

def generate_test_vectors():
    """Generate C-header files for all supported tests."""
    all_tests = [ ml_kem_keygen_params, ml_kem_encap_params, ml_kem_decap_params ]
#all_tests = [ml_kem_keygen_params, ml_kem_encap_params, ml_kem_decap_params ]
    update_tests(all_tests)
    for test in all_tests:
        generate_vectors_file(test)

def main():
    generate_test_vectors()

if __name__ == '__main__':
    main()



#!/usr/bin/python
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.


import os
import sys
import datetime
import shutil
import glob
from optparse import OptionParser
from subprocess import check_call

prinit_h = "pr/include/prinit.h"
f_conf = "configure"
f_conf_in = "configure.in"

def check_call_noisy(cmd, *args, **kwargs):
    print("Executing command:", cmd)
    check_call(cmd, *args, **kwargs)

o = OptionParser(usage="client.py [options] remove_beta | set_beta | print_library_versions | set_version_to_minor_release | set_version_to_patch_release | create_nspr_release_archive")

try:
    options, args = o.parse_args()
    action = args[0]
except IndexError:
    o.print_help()
    sys.exit(2)

def exit_with_failure(what):
    print("failure: ", what)
    sys.exit(2)

def check_files_exist():
    if (not os.path.exists(prinit_h)):
        exit_with_failure("cannot find expected header files, must run from inside NSPR hg directory")

def sed_inplace(sed_expression, filename):
    backup_file = filename + '.tmp'
    check_call_noisy(["sed", "-i.tmp", sed_expression, filename])
    os.remove(backup_file)

def toggle_beta_status(is_beta):
    check_files_exist()
    if (is_beta):
        print("adding Beta status to version numbers")
        sed_inplace('s/^\\(#define *PR_VERSION *\"[0-9.]\\+\\)\" *$/\\1 Beta\"/', prinit_h)
        sed_inplace('s/^\\(#define *PR_BETA *\\)PR_FALSE *$/\\1PR_TRUE/', prinit_h)

    else:
        print("removing Beta status from version numbers")
        sed_inplace('s/^\\(#define *PR_VERSION *\"[0-9.]\\+\\) *Beta\" *$/\\1\"/', prinit_h)
        sed_inplace('s/^\\(#define *PR_BETA *\\)PR_TRUE *$/\\1PR_FALSE/', prinit_h)
    print("please run 'hg stat' and 'hg diff' to verify the files have been verified correctly")

def print_beta_versions():
    check_call_noisy(["egrep", "#define *PR_VERSION|#define *PR_BETA", prinit_h])

def remove_beta_status():
    print("--- removing beta flags. Existing versions were:")
    print_beta_versions()
    toggle_beta_status(False)
    print("--- finished modifications, new versions are:")
    print_beta_versions()

def set_beta_status():
    print("--- adding beta flags. Existing versions were:")
    print_beta_versions()
    toggle_beta_status(True)
    print("--- finished modifications, new versions are:")
    print_beta_versions()

def print_library_versions():
    check_files_exist()
    check_call_noisy(["egrep", "#define *PR_VERSION|#define PR_VMAJOR|#define *PR_VMINOR|#define *PR_VPATCH|#define *PR_BETA", prinit_h])

def ensure_arguments_after_action(how_many, usage):
    if (len(sys.argv) != (2+how_many)):
        exit_with_failure("incorrect number of arguments, expected parameters are:\n" + usage)

def set_major_versions(major):
    sed_inplace('s/^\\(#define *PR_VMAJOR *\\).*$/\\1' + major + '/', prinit_h)
    sed_inplace('s/^MOD_MAJOR_VERSION=.*$/MOD_MAJOR_VERSION=' + major + '/', f_conf)
    sed_inplace('s/^MOD_MAJOR_VERSION=.*$/MOD_MAJOR_VERSION=' + major + '/', f_conf_in)

def set_minor_versions(minor):
    sed_inplace('s/^\\(#define *PR_VMINOR *\\).*$/\\1' + minor + '/', prinit_h)
    sed_inplace('s/^MOD_MINOR_VERSION=.*$/MOD_MINOR_VERSION=' + minor + '/', f_conf)
    sed_inplace('s/^MOD_MINOR_VERSION=.*$/MOD_MINOR_VERSION=' + minor + '/', f_conf_in)

def set_patch_versions(patch):
    sed_inplace('s/^\\(#define *PR_VPATCH *\\).*$/\\1' + patch + '/', prinit_h)
    sed_inplace('s/^MOD_PATCH_VERSION=.*$/MOD_PATCH_VERSION=' + patch + '/', f_conf)
    sed_inplace('s/^MOD_PATCH_VERSION=.*$/MOD_PATCH_VERSION=' + patch + '/', f_conf_in)

def set_full_lib_versions(version):
    sed_inplace('s/^\\(#define *PR_VERSION *\"\\)\\([0-9.]\\+\\)\\(.*\\)$/\\1' + version + '\\3/', prinit_h)

def set_all_lib_versions(version, major, minor, patch):
    set_full_lib_versions(version)
    set_major_versions(major)
    set_minor_versions(minor)
    set_patch_versions(patch)
    print()
    print("===========================")
    print("======== ATTENTION ========")
    print()
    print("You *MUST* manually edit file pr/tests/vercheck.c")
    print()
    print("Edit two arrays, named compatible_version and incompatible_version")
    print("according to the new version you're adding.")
    print()
    print("======== ATTENTION ========")
    print("===========================")

def set_version_to_minor_release():
    ensure_arguments_after_action(2, "major_version  minor_version")
    major = args[1].strip()
    minor = args[2].strip()
    version = major + '.' + minor
    patch = "0"
    set_all_lib_versions(version, major, minor, patch)

def set_version_to_patch_release():
    ensure_arguments_after_action(3, "major_version  minor_version  patch_release")
    major = args[1].strip()
    minor = args[2].strip()
    patch = args[3].strip()
    version = major + '.' + minor + '.' + patch
    set_all_lib_versions(version, major, minor, patch)

def create_nspr_release_archive():
    ensure_arguments_after_action(2, "nspr_release_version  nspr_hg_release_tag")
    nsprrel = args[1].strip() #e.g. 4.10.9
    nsprreltag = args[2].strip() #e.g. NSPR_4_10_9_RTM

    nspr_tar = "nspr-" + nsprrel + ".tar.gz"
    nspr_stagedir="../stage/v" + nsprrel + "/src"
    if (os.path.exists(nspr_stagedir)):
        exit_with_failure("nspr stage directory already exists: " + nspr_stagedir)

    check_call_noisy(["mkdir", "-p", nspr_stagedir])
    check_call_noisy(["hg", "archive", "-r", nsprreltag, "--prefix=nspr-" + nsprrel + "/nspr",
                      "../stage/v" + nsprrel + "/src/" + nspr_tar, "-X", ".hgtags"])
    print("changing to directory " + nspr_stagedir)
    os.chdir(nspr_stagedir)

    check_call("sha1sum " + nspr_tar + " > SHA1SUMS", shell=True)
    check_call("sha256sum " + nspr_tar + " > SHA256SUMS", shell=True)
    print("created directory " + nspr_stagedir + " with files:")
    check_call_noisy(["ls", "-l"])

if action in ('remove_beta'):
    remove_beta_status()

elif action in ('set_beta'):
    set_beta_status()

elif action in ('print_library_versions'):
    print_library_versions()

# x.y version number - 2 parameters
elif action in ('set_version_to_minor_release'):
    set_version_to_minor_release()

# x.y.z version number - 3 parameters
elif action in ('set_version_to_patch_release'):
    set_version_to_patch_release()

elif action in ('create_nspr_release_archive'):
    create_nspr_release_archive()

else:
    o.print_help()
    sys.exit(2)

sys.exit(0)



#!/usr/bin/env python
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

import argparse
import hashlib
import os
import threading

import frida

ARGUMENTS = {}
SOCKETS = {}
TERMINATE = threading.Condition()


def store_for_target(target, data):
    filename = hashlib.sha1(data).hexdigest()
    directory = os.path.join(ARGUMENTS["output"], target)

    os.makedirs(directory, exist_ok=True)

    with open(os.path.join(directory, filename), "wb") as f:
        f.write(data)


# --- asn1 ---


def on_SEC_ASN1DecodeItem_Util(payload):
    if not "data" in payload:
        return

    store_for_target("asn1", bytes(payload["data"].values()))


# --- certDN ---


def on_CERT_AsciiToName(payload):
    if not "data" in payload:
        return

    store_for_target("certDN", payload["data"].encode())


# --- ech ----


def on_tls13_DecodeEchConfigs(payload):
    if not "data" in payload:
        return

    store_for_target("ech", bytes(payload["data"].values()))


# --- pkcs7 ---


def on_CERT_DecodeCertPackage(payload):
    if not "data" in payload:
        return

    store_for_target("pkcs7", bytes(payload["data"].values()))


# --- pkcs8 ---


def on_PK11_ImportDERPrivateKeyInfoAndReturnKey(payload):
    if not "data" in payload:
        return

    store_for_target("pkcs8", bytes(payload["data"].values()))


# --- pkcs12 ---


def on_SEC_PKCS12DecoderUpdate(payload):
    if not "data" in payload:
        return

    store_for_target("pkcs12", bytes(payload["data"].values()))


# --- quickder ---


def on_SEC_QuickDERDecodeItem_Util(payload):
    if not "data" in payload:
        return

    store_for_target("quickder", bytes(payload["data"].values()))


# --- smime ---


def on_NSS_CMSDecoder_Update(payload):
    if not "data" in payload:
        return

    store_for_target("smime", bytes(payload["data"].values()))


# --- TLS ---


def on_ssl_DefClose(payload):
    ss = payload["ss"]
    if not ss in SOCKETS:
        return

    # There is no way for us to determine (in a clean and future-proof)
    # way the variant (DTLS/TLS) and origin (client/server) of the
    # received data.
    # Since you want to minimize the corpus anyway, we just say it belongs
    # to all possible targets.
    data = SOCKETS[ss].lstrip(b"\x00")
    store_for_target("tls-client", data)

    base_output_path = os.path.abspath(ARGUMENTS["output"])
    for target in ["dtls-client", "dtls-server", "tls-client", "tls-server"]:
        if not os.path.exists(os.path.join(base_output_path, target)):
            os.symlink(os.path.join(base_output_path, "tls-client"),
                       os.path.join(base_output_path, target),
                       target_is_directory=True)

    del SOCKETS[ss]


def on_ssl_DefRecv(payload):
    if not "data" in payload:
        return

    ss = payload["ss"]
    if not ss in SOCKETS:
        SOCKETS[ss] = bytes()

    SOCKETS[ss] += bytes(payload["data"].values())


def on_ssl_DefRead(payload):
    if not "data" in payload:
        return

    ss = payload["ss"]
    if not ss in SOCKETS:
        SOCKETS[ss] = bytes()

    SOCKETS[ss] += bytes(payload["data"].values())


def script_on_message(message, _data):
    if message["type"] != "send":
        print(message)
        return

    assert message["type"] == "send"

    payload = message["payload"]
    func = "on_" + payload["func"]

    assert func in globals()
    globals()[func](payload)


def session_on_detached():
    with TERMINATE:
        TERMINATE.notify_all()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--script", required=True, type=str)
    parser.add_argument("--nss-build",
                        required=True,
                        type=str,
                        help="e.g. /path/to/dist/Debug")
    parser.add_argument("--program", required=True, type=str)
    parser.add_argument("--output", required=True, type=str)

    args, programargs = parser.parse_known_args()

    global ARGUMENTS
    ARGUMENTS = vars(args)

    with open(args.script, "r") as f:
        script = f.read()

    pid = frida.spawn(program=args.program,
                      argv=programargs,
                      env={
                          **os.environ, "LD_LIBRARY_PATH":
                          os.path.join(args.nss_build, "lib")
                      })
    session = frida.attach(pid)

    script = session.create_script(script)
    script.load()

    script.on("message", script_on_message)

    session.resume()
    frida.resume(pid)

    session.on("detached", session_on_detached)

    with TERMINATE:
        TERMINATE.wait()


if __name__ == "__main__":
    main()
